import datetime

from django.core.management import call_command
from django.test import TestCase

from django_ses.management.commands import get_ses_statistics as mod_get_ses_statistics
from django_ses.models import SESStat

data_points = [
    {
        'Complaints': '1',
        'Timestamp': datetime.datetime(year=2012, month=1, day=1, hour=2),
        'DeliveryAttempts': '2',
        'Bounces': '3',
        'Rejects': '4'
    },
    {
        'Complaints': '1',
        'Timestamp': datetime.datetime(year=2012, month=1, day=3, hour=2),
        'DeliveryAttempts': '2',
        'Bounces': '3',
        'Rejects': '4'
    },
    {
        'Complaints': '1',
        'Timestamp': datetime.datetime(year=2012, month=1, day=3, hour=3),
        'DeliveryAttempts': '2',
        'Bounces': '3',
        'Rejects': '4'
    }
]


class FakeSESConnection:
    def __init__(self, *args, **kwargs):
        pass

    def get_send_statistics(self):
        return {
            'SendDataPoints': data_points,
            'ResponseMetadata': {
                'RequestId': '1'
            },
        }


class SESCommandTest(TestCase):

    def setUp(self):
        mod_get_ses_statistics.boto3.client = FakeSESConnection

    def test_get_statistics(self):
        # Test the get_ses_statistics management command
        call_command('get_ses_statistics')

        # Test that days with a single data point is saved properly
        stat = SESStat.objects.get(date='2012-01-01')
        self.assertEqual(stat.complaints, 1)
        self.assertEqual(stat.delivery_attempts, 2)
        self.assertEqual(stat.bounces, 3)
        self.assertEqual(stat.rejects, 4)

        # Test that days with multiple data points get saved properly
        stat = SESStat.objects.get(date='2012-01-03')
        self.assertEqual(stat.complaints, 2)
        self.assertEqual(stat.delivery_attempts, 4)
        self.assertEqual(stat.bounces, 6)
        self.assertEqual(stat.rejects, 8)

        # Changing data points should update database records too
        data_points[0]['Complaints'] = '2'
        data_points[0]['DeliveryAttempts'] = '3'
        data_points[0]['Bounces'] = '4'
        data_points[0]['Rejects'] = '5'

        call_command('get_ses_statistics')
        stat = SESStat.objects.get(date='2012-01-01')
        self.assertEqual(stat.complaints, 2)
        self.assertEqual(stat.delivery_attempts, 3)
        self.assertEqual(stat.bounces, 4)
        self.assertEqual(stat.rejects, 5)



# -*- coding: utf-8 -*-
# Generated by Django 1.9.4 on 2016-04-27 12:52


from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='SESStat',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('date', models.DateField(db_index=True, unique=True)),
                ('delivery_attempts', models.PositiveIntegerField()),
                ('bounces', models.PositiveIntegerField()),
                ('complaints', models.PositiveIntegerField()),
                ('rejects', models.PositiveIntegerField()),
            ],
            options={
                'verbose_name': 'SES Stat',
                'ordering': ['-date'],
            },
        ),
    ]



try:
    from unittest import mock
except ImportError:
    import mock

try:
    import requests
except ImportError:
    requests = None

try:
    from cryptography import x509
except ImportError:
    x509 = None

from unittest import TestCase, skipIf

from django_ses.utils import BounceMessageVerifier, clear_cert_cache


class BounceMessageVerifierTest(TestCase):
    """
    Test for bounce message signature verification
    """

    simple_msg = {
        "Message": "Message text",
        "MessageId": "Some ID",
        "Subject": "Equity for all",
        "Timestamp": "Yesterday",
        "TopicArn": "arn:aws...",
        "Type": "Notification",
    }

    VALID_CERT = (b"-----BEGIN CERTIFICATE-----\nMIIF1zCCBL+gAwIBAgIQB9pYWG3Mi7xej22g9pobJTANBgkqhkiG9w0BAQsFADBG\n"
                  b"MQswCQYDVQQGEwJVUzEPMA0GA1UEChMGQW1hem9uMRUwEwYDVQQLEwxTZXJ2ZXIg\nQ0EgMUIxDzANBgNVBAMTBkFtYXpvb"
                  b"jAeFw0yMTA5MDcwMDAwMDBaFw0yMjA4MTcy\nMzU5NTlaMBwxGjAYBgNVBAMTEXNucy5hbWF6b25hd3MuY29tMIIBIjANBg"
                  b"kqhkiG\n9w0BAQEFAAOCAQ8AMIIBCgKCAQEAutFqueT3XgP13udzxE6UpbdjOtVO5DwoMpSM\niDNMnGzF1TYH5/R2LPUOB"
                  b"eTB0SkKnR4kpNcUZhicpGD4aKciz/GEZ6wu65xncfT9\nH/KBOQwoXYTuClHwp6fYpGzcGFaFoEYMnijL/o4qmTSd+ukglQ"
                  b"UgKpsDw4ofw6rU\nm2CttJo+GQSNQ9NfGR1h/0J+zsApkeSYrXRx5wNlu87z8os1C/6PBrUHwt3xXeaf\nXzfwut8aRRYsS"
                  b"8BySOA9DAgLfNHlfdQCjKPXKrG/ussgReyWD6n/HH+j7Uha3xos\nTzQqJifcxlTq6MxWdPR6fDaJNvqw6DOE7UjUNxHguX"
                  b"HlVfxhlQIDAQABo4IC6TCC\nAuUwHwYDVR0jBBgwFoAUWaRmBlKge5WSPKOUByeWdFv5PdAwHQYDVR0OBBYEFAqz\nC+vyo"
                  b"uneE7mWWLbi9i0UsWUbMBwGA1UdEQQVMBOCEXNucy5hbWF6b25hd3MuY29t\nMA4GA1UdDwEB/wQEAwIFoDAdBgNVHSUEFj"
                  b"AUBggrBgEFBQcDAQYIKwYBBQUHAwIw\nOwYDVR0fBDQwMjAwoC6gLIYqaHR0cDovL2NybC5zY2ExYi5hbWF6b250cnVzdC5"
                  b"j\nb20vc2NhMWIuY3JsMBMGA1UdIAQMMAowCAYGZ4EMAQIBMHUGCCsGAQUFBwEBBGkw\nZzAtBggrBgEFBQcwAYYhaHR0cD"
                  b"ovL29jc3Auc2NhMWIuYW1hem9udHJ1c3QuY29t\nMDYGCCsGAQUFBzAChipodHRwOi8vY3J0LnNjYTFiLmFtYXpvbnRydXN"
                  b"0LmNvbS9z\nY2ExYi5jcnQwDAYDVR0TAQH/BAIwADCCAX0GCisGAQQB1nkCBAIEggFtBIIBaQFn\nAHYAKXm+8J45OSHwVn"
                  b"OfY6V35b5XfZxgCvj5TV0mXCVdx4QAAAF7vfDVkQAABAMA\nRzBFAiEA2XfHuy36aqRFiaL8c3md2mH451go8707+fRE0pE"
                  b"dSRACIE/g5FXTUXUZ\nPFcmOhm9TZ+uMY1i4CIQ/CKVWln6C3t+AHYAUaOw9f0BeZxWbbg3eI8MpHrMGyfL\n956IQpoN/t"
                  b"SLBeUAAAF7vfDVjAAABAMARzBFAiBF1MhhFP0+FQt3daDFfMYoWwnr\nmuTInrjNpwfzlvQBugIhAPYadFzr+LaxSJoiZEb"
                  b"EHBvTts7bT0M3eCQONA2O7w6n\nAHUAQcjKsd8iRkoQxqE6CUKHXk4xixsD6+tLx2jwkGKWBvYAAAF7vfDVdAAABAMA\nRj"
                  b"BEAiAtPapmFAuA71ih4NoSd5hJelzAltNQpxDMcDfDyHyU8gIgWxmaa6+2KbBu\n9xdv379zvnJACFR7jc+4asl08Dn4aag"
                  b"wDQYJKoZIhvcNAQELBQADggEBAA54QX0u\noFWXfMmv02CGZv4NWo5TapyeeixQ2kKpZHRdVZjxZrw+hoF6HD7P3kGjH8zt"
                  b"yJll\ntDxB0qgMltbPhQdScwhA6iTgoaBYqEUC/VHKd4PmmPT6yIsM36NBZVmkGlzl5uNo\n/dBgBaG0SsVJnhr5zro3c2q"
                  b"uC7n6fVGEZhf/UgQwRnnvThnvbNKguglDMq4uEqv8\nnjKyleht+glkcmXO0m9qLKt6BOS0amy6U2GlAwRn0Wx02ndJtnRC"
                  b"SC6kPuRWK/SQ\nFEjB7gCK4hdKaAOuWdZpI55vF6ifOeM8toC3g7ofO8qLTnJupAG+ZitY5J3cvHWr\nHqOUdKigPDHYLRo"
                  b"=\n-----END CERTIFICATE-----\n")
    # Any changes to this message will break the signature validity test.
    valid_msg = {
        "Type": "Notification",
        "MessageId": "97903a0c-8818-5442-94b2-60b0c63d3da0",
        "TopicArn": "arn:aws:sns:us-east-1:364852123998:test-email",
        "Subject": "Amazon SES Email Event Notification",
        "Message": '{"eventType":"Complaint","complaint":{"feedbackId":"0100017fd2f87864-7f070191-4eb1-49e3-a42e-'
                   '7e996b718724-000000","complaintSubType":null,"complainedRecipients":[{"emailAddress":'
                   '"complaint@simulator.amazonses.com"}],"timestamp":"2022-03-28T23:59:33.564Z","userAgent":'
                   '"Amazon SES Mailbox Simulator","complaintFeedbackType":"abuse","arrivalDate":'
                   '"2022-03-28T23:59:33.564Z"},"mail":{"timestamp":"2022-03-28T23:59:32.804Z","source":'
                   '"jesus.islasf@alumno.buap.mx","sourceArn":"arn:aws:ses:us-east-1:364852123998:'
                   'identity/jesus.islasf@alumno.buap.mx","sendingAccountId":"364852123998","messageId":'
                   '"0100017fd2f875c4-c2929163-2767-49b1-8513-1a29e428e134-000000","destination":'
                   '["complaint@simulator.amazonses.com"],"headersTruncated":false,"headers":[{"name":"From",'
                   '"value":"jesus.islasf@alumno.buap.mx"},{"name":"To","value":"complaint@simulator.amazonses.com"},'
                   '{"name":"Subject","value":"Complaint Notification"},{"name":"MIME-Version","value":"1.0"},'
                   '{"name":"Content-Type","value":"multipart/alternative;  '
                   'boundary=\\"----=_Part_1160373_1543555432.1648511972808\\""}],"commonHeaders":{"from":'
                   '["jesus.islasf@alumno.buap.mx"],"to":["complaint@simulator.amazonses.com"],"messageId":'
                   '"0100017fd2f875c4-c2929163-2767-49b1-8513-1a29e428e134-000000","subject":'
                   '"Complaint Notification"},"tags":{"ses:operation":["SendEmail"],"ses:configuration-set":'
                   '["test-set"],"ses:source-ip":["189.203.131.80"],"ses:from-domain":["alumno.buap.mx"],'
                   '"ses:caller-identity":["root"]}}}\n',
        "Timestamp": "2022-03-28T23:59:33.713Z",
        "SignatureVersion": "1",
        "Signature": "MAnUSvI5C6S0GERh05oFiMtpZuGTW0J/6I/AaBinsZWK+Na8TwJsiNSUjShgZ8NItzQkjBnY1R1qT0wtPf6FfNXxGu3"
                     "oQRaYsj1alz5NJyiuZwGlryX9LHuOoSJ7tGhrKA5yfYI1JPZsdUJKnkI3+UgKbIg7ml2FoSMU8s3HP3V/FOhvp6V5P6y"
                     "t2okxABbw13WQPrzeUCZ9pRLgB3TnY59wsWM2SlynWEG/u/pFHyzuvkmtrGZtjZfITm7bfMnGy8FTOox4PfzCu4bysKl"
                     "Ubhc/yJ0fzI/+XsT2gKasXETzlmx6vd4qKgKWP5U9OJVh+Cx//npFDCBI2Tba8JK+Cg==",
        "SigningCertURL":
            "https://sns.us-east-1.amazonaws.com/SimpleNotificationService-7ff5318490ec183fbaddaa2a969abfda.pem",
        "UnsubscribeURL": "https://sns.us-east-1.amazonaws.com/?Action=Unsubscribe&SubscriptionArn=arn:aws:sns:"
                          "us-east-1:364852123998:test-email:0fe0bffc-6470-4502-9414-d5e3d9fdd71e",
    }

    valid_msg_missing_fields = {
        "Type": "Notification",
        "MessageId": "71a6b82e-0f1a-58db-b902-e2ef60f87541",
        "TopicArn": "arn:aws:sns:us-east-1:364852123998:test-email",
        # Normally you'd see a subject key/value pair here, but not on this
        # notification.
        "Message": '{"notificationType":"Bounce","bounce":{"feedbackId":"0100017fe7044080-b95aeb18-3fb8-408a-be9b-'
                   '15fdbac3945a-000000","bounceType":"Permanent","bounceSubType":"General","bouncedRecipients":['
                   '{"emailAddress":"bounce@simulator.amazonses.com","action":"failed","status":"5.1.1",'
                   '"diagnosticCode":"smtp; 550 5.1.1 user unknown"}],"timestamp":"2022-04-01T21:24:49.000Z",'
                   '"remoteMtaIp":"3.231.136.178","reportingMTA":"dns; a48-30.smtp-out.amazonses.com"},"mail":'
                   '{"timestamp":"2022-04-01T21:24:49.422Z","source":"jesus.islasf@alumno.buap.mx","sourceArn":'
                   '"arn:aws:ses:us-east-1:364852123998:identity/jesus.islasf@alumno.buap.mx","sourceIp":'
                   '"189.203.131.80","sendingAccountId":"364852123998","messageId":'
                   '"0100017fe7043e8e-fc9c357d-a9a4-49bc-a4b4-fa984009d568-000000","destination":'
                   '["bounce@simulator.amazonses.com"],"headersTruncated":false,"headers":[{"name":'
                   '"From","value":"jesus.islasf@alumno.buap.mx"},{"name":"To","value":'
                   '"bounce@simulator.amazonses.com"},{"name":"Subject","value":"Subject Test"},'
                   '{"name":"MIME-Version","value":"1.0"},{"name":"Content-Type","value":'
                   '"multipart/alternative;  boundary=\\"----=_Part_2196631_1474166380.1648848289426\\""}],'
                   '"commonHeaders":{"from":["jesus.islasf@alumno.buap.mx"],"to":["bounce@simulator.amazonses.com"],'
                   '"subject":"Subject Test"}}}',
        "Timestamp": "2022-04-01T21:24:50.200Z",
        "SignatureVersion": "1",
        "Signature": "K27nwTcT0qTPKEQP3noOWV21gDGB6XnLXSwN2i+4176naErwyDSd72w44UesYE/KaRXU+Kusi7b8uoLYPOcYXHH45UCFMrSt"
                     "f9nzu0uIZdMSd7cFGPm0KAxqoDcP9UQw0+ssK1rjVWkywTYmeDyFF/j3IQZxTA/vINOLYbrmMKhyJUPjcZZwdgLmlKcNfKJ5"
                     "PKmg5WXlr8nWtjW3K+k725nkoAZemuAFt3PmA2k35JoHphkcOBjV2f1qR9zJTOgrVQ1d6k2v6t8G7Nlg6FP5OiwThgKHkehI"
                     "PfJfLhTmo05tfPCBzXYMzDbnX+HLidvkyibHlalRl/DuDxXXL7SUiA==",
        "SigningCertURL": "https://sns.us-east-1.amazonaws.com/SimpleNotificationService-"
                          "7ff5318490ec183fbaddaa2a969abfda.pem",
        "UnsubscribeURL": "https://sns.us-east-1.amazonaws.com/?Action=Unsubscribe&SubscriptionArn="
                          "arn:aws:sns:us-east-1:364852123998:test-email:0fe0bffc-6470-4502-9414-d5e3d9fdd71e",
    }

    def tearDown(self):
        # Reset the cache after each test
        clear_cert_cache()

    @skipIf(requests is None, "requests is not installed")
    @skipIf(x509 is None, "cryptography is not installed")
    def test_load_certificate(self):
        verifier = BounceMessageVerifier({})
        with mock.patch.object(verifier, "_get_cert_url") as get_cert_url:
            get_cert_url.return_value = "http://www.example.com/"
            with mock.patch.object(requests, "get") as request_get:
                request_get.return_value.status_code = 200
                request_get.return_value.content = "Spam"
                with mock.patch.object(
                    x509, "load_pem_x509_certificate"
                ) as load_cert_string:
                    self.assertEqual(
                        verifier.certificate, load_cert_string.return_value
                    )

    @skipIf(requests is None, "requests is not installed")
    @skipIf(x509 is None, "cryptography is not installed")
    def test_valid_msg_validates(self):
        """Does a valid message get validated properly?"""
        verifier = BounceMessageVerifier(self.valid_msg)
        with mock.patch.object(requests, "get") as request_get:
            request_get.return_value.content = self.VALID_CERT
            request_get.return_value.status_code = 200
            self.assertTrue(verifier.is_verified())

    @skipIf(requests is None, "requests is not installed")
    @skipIf(x509 is None, "cryptography is not installed")
    def test_valid_msg_missing_fields_validates(self):
        """Does a valid message with missing fields get validated properly?"""
        verifier = BounceMessageVerifier(self.valid_msg_missing_fields)
        with mock.patch.object(requests, "get") as request_get:
            request_get.return_value.content = self.VALID_CERT
            request_get.return_value.status_code = 200
            self.assertTrue(verifier.is_verified())

    @skipIf(requests is None, "requests is not installed")
    @skipIf(x509 is None, "cryptography is not installed")
    def test_invalid_message_fails(self):
        """Does an invalid message fail validation?"""
        verifier = BounceMessageVerifier(self.simple_msg)
        self.assertFalse(verifier.is_verified())

    @skipIf(requests is None, "requests is not installed")
    @skipIf(x509 is None, "cryptography is not installed")
    def test_cert_is_cached(self):
        """Does the certificate get cached properly?"""
        verifier = BounceMessageVerifier(self.valid_msg)
        with mock.patch.object(requests, "get") as request_get, mock.patch.object(
            x509, "load_pem_x509_certificate"
        ):
            request_get.return_value.content = b"Spam"
            request_get.return_value.status_code = 200
            verifier.certificate
            verifier.certificate
            request_get.assert_called_once()

    def test_get_cert_url(self):
        """
        Test url trust verification
        """
        cert_url = (
            "https://sns.test-example.amazonaws.com/SimpleNotificationService-abcd.pem"
        )
        verifier = BounceMessageVerifier(
            {
                "SigningCertURL": cert_url,
            }
        )
        self.assertEqual(verifier._get_cert_url(), cert_url)

    def test_http_cert_url(self):
        """
        Test url trust verification. Non-https urls should be rejected.
        """
        verifier = BounceMessageVerifier(
            {
                "SigningCertURL": "http://amazonaws.com/",
            }
        )
        self.assertEqual(verifier._get_cert_url(), None)

    def test_untrusted_cert_url_domain(self):
        """
        Test url trust verification. Untrusted domains should be rejected.
        """
        verifier = BounceMessageVerifier(
            {
                "SigningCertURL": "https://www.example.com/",
            }
        )
        self.assertEqual(verifier._get_cert_url(), None)

    def test_get_bytes_to_sign(self):
        verifier = BounceMessageVerifier(self.simple_msg)
        actual_result = verifier._get_bytes_to_sign()
        correct_result = (
            b"Message\nMessage text\n"
            b"MessageId\nSome ID\n"
            b"Subject\nEquity for all\n"
            b"Timestamp\nYesterday\n"
            b"TopicArn\narn:aws...\n"
            b"Type\nNotification\n"
        )
        self.assertEqual(actual_result, correct_result)
        self.assertTrue(isinstance(actual_result, bytes))



from django.conf import settings
from django.test import TestCase

from tests.utils import unload_django_ses


class SettingsImportTest(TestCase):
    def test_aws_access_key_given(self):
        settings.AWS_ACCESS_KEY_ID = "Yjc4MzQ4MGYzMTBhOWY3ODJhODhmNTBkN2QwY2IyZTdhZmU1NDM1ZQo"
        settings.AWS_SECRET_ACCESS_KEY = "NTBjYzAzNzVlMTA0N2FiMmFlODlhYjY5OTYwZjNkNjZmMWNhNzRhOQo"
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.ACCESS_KEY, settings.AWS_ACCESS_KEY_ID)
        self.assertEqual(django_ses.settings.SECRET_KEY, settings.AWS_SECRET_ACCESS_KEY)

    def test_ses_access_key_given(self):
        settings.AWS_SES_ACCESS_KEY_ID = "YmM2M2QwZTE3ODk3NTJmYzZlZDc1MDY0ZmJkMDZjZjhmOTU0MWQ4MAo"
        settings.AWS_SES_SECRET_ACCESS_KEY = "NDNiMzRjNzlmZGU0ZDAzZTQxNTkwNzdkNWE5Y2JlNjk4OGFkM2UyZQo"
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.ACCESS_KEY, settings.AWS_SES_ACCESS_KEY_ID)
        self.assertEqual(django_ses.settings.SECRET_KEY, settings.AWS_SES_SECRET_ACCESS_KEY)

    def test_aws_session_token_given(self):
        settings.AWS_SESSION_TOKEN = "FwoGZXIvYXdzED8aDAILqEtZvcDCx+KsFCK1AUwcLbm4d+mAlRWYN+r1adKoIfw"
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.SESSION_TOKEN, settings.AWS_SESSION_TOKEN)

    def test_ses_session_token_given(self):
        settings.AWS_SES_SESSION_TOKEN = "jQYyLYI7nmsYjpQa2aynxovr7rwKrj71PQstMbK2oKwaT1FzasM0hjs+C5uLh"
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.SESSION_TOKEN, settings.AWS_SES_SESSION_TOKEN)

    def test_ses_configuration_set_given(self):
        settings.AWS_SES_CONFIGURATION_SET = "test-set"
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.AWS_SES_CONFIGURATION_SET, settings.AWS_SES_CONFIGURATION_SET)

    def test_ses_region_to_endpoint_default_given(self):
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.AWS_SES_REGION_NAME, 'us-east-1')
        self.assertEqual(django_ses.settings.AWS_SES_REGION_ENDPOINT,
                         f'email.{django_ses.settings.AWS_SES_REGION_NAME}.amazonaws.com')

    def test_ses_region_to_endpoint_set_given(self):
        settings.AWS_SES_REGION_NAME = 'eu-west-1'
        unload_django_ses()
        import django_ses
        self.assertEqual(django_ses.settings.AWS_SES_REGION_NAME, 'eu-west-1')
        self.assertEqual(django_ses.settings.AWS_SES_REGION_ENDPOINT, 'email.eu-west-1.amazonaws.com')



import base64
import logging
import re
import warnings
from builtins import bytes
from urllib.error import URLError
from urllib.parse import urlparse
from urllib.request import urlopen

from django.core.exceptions import ImproperlyConfigured

from django_ses import settings
from django_ses.deprecation import RemovedInDjangoSES20Warning

logger = logging.getLogger(__name__)

_CERT_CACHE = {}

SES_REGEX_CERT_URL = re.compile(
    r"(?i)^https://sns\.[a-z0-9\-]+\.amazonaws\.com(\.cn)?/SimpleNotificationService\-[a-z0-9]+\.pem$"
)


def clear_cert_cache():
    """Clear the certificate cache.

    This one-liner exists to discourage imports and direct usage of
    _CERT_CACHE.

    :returns None
    """
    _CERT_CACHE.clear()


class EventMessageVerifier(object):
    """
    A utility class for validating event messages

    See: http://docs.amazonwebservices.com/sns/latest/gsg/SendMessageToHttp.verify.signature.html
    """

    _REQ_DEP_TMPL = (
        "%s is required for event message verification. Please install "
        "`django-ses` with the `event` extra - e.g. "
        "`pip install django-ses[events]`."
    )

    def __init__(self, notification):
        """
        Creates a new event message from the given dict.
        """
        self._data = notification
        self._verified = None

    def is_verified(self):
        """
        Verifies an SES event message.

        Sign the bytes from the notification and compare it to the signature in
        the notification. If same, return True; else False.
        """
        if self._verified is not None:
            return self._verified

        signature = self._data.get("Signature")
        if not signature:
            self._verified = False
            return self._verified

        # Decode the signature from base64
        signature = bytes(base64.b64decode(signature))

        # Get the message to sign
        sign_bytes = self._get_bytes_to_sign()
        if not sign_bytes:
            self._verified = False
            return self._verified

        if not self.certificate:
            self._verified = False
            return self._verified

        try:
            from cryptography.exceptions import InvalidSignature
            from cryptography.hazmat.primitives import hashes
            from cryptography.hazmat.primitives.asymmetric import padding
        except ImportError:
            raise ImproperlyConfigured(self._REQ_DEP_TMPL % "`cryptography`")

        # Extract the public key
        pkey = self.certificate.public_key()

        # Use the public key to verify the signature.
        try:
            # The details here do not appear to be documented, but the
            # algorithm and padding choices work in testing, which should mean
            # they're the right ones.
            pkey.verify(
                signature,
                sign_bytes,
                padding.PKCS1v15(),
                hashes.SHA1(),
            )
        except InvalidSignature:
            logger.warning(
                "Invalid signature on message with ID: %s",
                self._data.get("MessageId"),
            )
            self._verified = False
        else:
            self._verified = True
        return self._verified

    @property
    def certificate(self):
        """
        Retrieves the certificate used to sign the event message.

        :returns: None if the cert cannot be retrieved. Else, gets the cert
        caches it, and returns it, or simply returns it if already cached.
        """
        cert_url = self._get_cert_url()
        if not cert_url:
            return None

        if cert_url in _CERT_CACHE:
            return _CERT_CACHE[cert_url]

        # Only load certificates from a certain domain?
        # Without some kind of trusted domain check, any old joe could
        # craft a event message and sign it using his own certificate
        # and we would happily load and verify it.
        try:
            import requests
            from requests import RequestException
        except ImportError:
            raise ImproperlyConfigured(self._REQ_DEP_TMPL % "`requests`")

        try:
            from cryptography import x509
        except ImportError:
            raise ImproperlyConfigured(self._REQ_DEP_TMPL % "`cryptography`")

        # We use requests because it verifies the https certificate when
        # retrieving the signing certificate. If https was somehow hijacked
        # then all bets are off.
        try:
            response = requests.get(cert_url, timeout=10)
            response.raise_for_status()
        except RequestException as exc:
            logger.warning(
                "Network error downloading certificate from " "%s: %s",
                cert_url,
                exc,
            )
            _CERT_CACHE[cert_url] = None
            return _CERT_CACHE[cert_url]

        # Handle errors loading the certificate.
        # If the certificate is invalid then return
        # false as we couldn't verify the message.
        try:
            _CERT_CACHE[cert_url] = x509.load_pem_x509_certificate(response.content)
        except ValueError as e:
            logger.warning('Could not load certificate from %s: "%s"', cert_url, e)
            _CERT_CACHE[cert_url] = None

        return _CERT_CACHE[cert_url]

    def _get_cert_url(self):
        """
        Get the signing certificate URL.
        Only accept urls that match the domains set in the
        AWS_SNS_EVENT_CERT_TRUSTED_DOMAINS setting. Sub-domains
        are allowed. i.e. if amazonaws.com is in the trusted domains
        then sns.us-east-1.amazonaws.com will match.
        """
        cert_url = self._data.get("SigningCertURL")
        if not cert_url:
            logger.warning('No signing certificate URL: "%s"', cert_url)
            return None

        if not cert_url.startswith("https://"):
            logger.warning('Untrusted certificate URL: "%s"', cert_url)
            return None

        url_obj = urlparse(cert_url)
        for trusted_domain in settings.EVENT_CERT_DOMAINS:
            parts = trusted_domain.split(".")
            if "amazonaws.com" in trusted_domain:
                if not SES_REGEX_CERT_URL.match(cert_url):
                    if len(parts) < 4:
                        return None
                    else:
                        logger.warning('Possible security risk for: "%s"', cert_url)
                        logger.warning(
                            "It is strongly recommended to configure the full domain in EVENT_CERT_DOMAINS. "
                            "See v3.5.0 release notes for more details."
                        )

            if url_obj.netloc.split(".")[-len(parts) :] == parts:
                return cert_url

        return None

    def _get_bytes_to_sign(self):
        """
        Creates the message used for signing SNS notifications.
        This is used to verify the bounce message when it is received.
        """

        # Depending on the message type the fields to add to the message
        # differ so we handle that here.
        msg_type = self._data.get("Type")
        if msg_type == "Notification":
            fields_to_sign = [
                "Message",
                "MessageId",
                "Subject",
                "Timestamp",
                "TopicArn",
                "Type",
            ]
        elif (
            msg_type == "SubscriptionConfirmation"
            or msg_type == "UnsubscribeConfirmation"
        ):
            fields_to_sign = [
                "Message",
                "MessageId",
                "SubscribeURL",
                "Timestamp",
                "Token",
                "TopicArn",
                "Type",
            ]
        else:
            # Unrecognized type
            logger.warning('Unrecognized SNS message Type: "%s"', msg_type)
            return None

        bytes_to_sign = []
        for field in fields_to_sign:
            field_value = self._data.get(field)
            if not field_value:
                continue

            # Some notification types do not have all fields. Only add fields
            # with values.
            bytes_to_sign.append(f"{field}\n{field_value}\n")

        return "".join(bytes_to_sign).encode()


def BounceMessageVerifier(*args, **kwargs):  # noqa: N802
    warnings.warn(
        "utils.BounceMessageVerifier is deprecated. It is renamed to EventMessageVerifier.",
        RemovedInDjangoSES20Warning,
    )

    # parameter name is renamed from bounce_dict to notification.
    if "bounce_dict" in kwargs:
        kwargs["notification"] = kwargs["bounce_dict"]
        del kwargs["bounce_dict"]

    return EventMessageVerifier(*args, **kwargs)


def verify_event_message(notification):
    """
    Verify an SES/SNS event notification message.
    """
    verifier = EventMessageVerifier(notification)
    return verifier.is_verified()


def verify_bounce_message(msg):
    """
    Verify an SES/SNS bounce(event) notification message.
    """
    warnings.warn(
        "utils.verify_bounce_message is deprecated. It is renamed to verify_event_message.",
        RemovedInDjangoSES20Warning,
    )
    return verify_event_message(msg)


def confirm_sns_subscription(notification):
    logger.info(
        "Received subscription confirmation: TopicArn: %s",
        notification.get("TopicArn"),
        extra={
            "notification": notification,
        },
    )

    # Get the subscribe url and hit the url to confirm the subscription.
    subscribe_url = notification.get("SubscribeURL")
    try:
        urlopen(subscribe_url).read()
    except URLError as e:
        # Some kind of error occurred when confirming the request.
        logger.error(
            'Could not confirm subscription: "%s"',
            e,
            extra={
                "notification": notification,
            },
            exc_info=True,
        )



#!/usr/bin/env python

from collections import defaultdict

import boto3
from django.core.management.base import BaseCommand

from django_ses import settings
from django_ses.models import SESStat
from django_ses.views import stats_to_list


def stat_factory():
    return {
        'delivery_attempts': 0,
        'bounces': 0,
        'complaints': 0,
        'rejects': 0,
    }


class Command(BaseCommand):
    """
    Get SES sending statistic and store the result, grouped by date.
    """

    def handle(self, *args, **options):

        connection = boto3.client(
            'ses',
            aws_access_key_id=settings.ACCESS_KEY,
            aws_secret_access_key=settings.SECRET_KEY,
            aws_session_token=settings.SESSION_TOKEN,
            region_name=settings.AWS_SES_REGION_NAME,
            endpoint_url=settings.AWS_SES_REGION_ENDPOINT_URL,
            config=settings.AWS_SES_CONFIG,
        )
        stats = connection.get_send_statistics()
        data_points = stats_to_list(stats, localize=False)
        stats_dict = defaultdict(stat_factory)

        for data in data_points:
            attempts = int(data['DeliveryAttempts'])
            bounces = int(data['Bounces'])
            complaints = int(data['Complaints'])
            rejects = int(data['Rejects'])
            date = data['Timestamp'].date()
            stats_dict[date]['delivery_attempts'] += attempts
            stats_dict[date]['bounces'] += bounces
            stats_dict[date]['complaints'] += complaints
            stats_dict[date]['rejects'] += rejects

        for k, v in stats_dict.items():
            stat, created = SESStat.objects.get_or_create(
                date=k,
                defaults={
                    'delivery_attempts': v['delivery_attempts'],
                    'bounces': v['bounces'],
                    'complaints': v['complaints'],
                    'rejects': v['rejects'],
                })

            # If statistic is not new, modify data if values are different
            if not created and stat.delivery_attempts != v['delivery_attempts']:
                stat.delivery_attempts = v['delivery_attempts']
                stat.bounces = v['bounces']
                stat.complaints = v['complaints']
                stat.rejects = v['rejects']
                stat.save()



from django.contrib import admin
from django.contrib.staticfiles.urls import staticfiles_urlpatterns
from django.urls import include, path

from django_ses.views import SESEventWebhookView
from example import views

admin.autodiscover()

urlpatterns = [
    '',
    path('admin/doc/', include('django.contrib.admindocs.urls')),
    path('admin/', include(admin.site.urls)),

    path('', views.index, name='index'),
    path('send-email/', views.send_email, name='send-email'),
    path('reporting/', include('django_ses.urls')),

    path('bounce/', 'django_ses.views.handle_bounce', name='handle_bounce'),  # Deprecated, see SESEventWebhookView
    path('event-webhook/', SESEventWebhookView.as_view(), name='event_webhook'),
]

urlpatterns += staticfiles_urlpatterns()



from django.urls import path

from django_ses.views import DashboardView, SESEventWebhookView, handle_bounce

urlpatterns = [
    path('dashboard/', DashboardView.as_view(), name='django_ses_stats'),
    path('bounce/', handle_bounce, name='django_ses_bounce'),
    path('event-webhook/', SESEventWebhookView.as_view(), name='event_webhook'),
]



from django.contrib import admin

from .models import SESStat


@admin.register(SESStat)
class SESStatAdmin(admin.ModelAdmin):
    list_display = ('date', 'delivery_attempts', 'bounces', 'complaints',
                    'rejects')



# -*- coding: utf-8 -*-

import email

from django.conf import settings as django_settings
from django.core.mail import send_mail
from django.test import TestCase
from django.utils.encoding import smart_str

import django_ses
from django_ses import settings

# random key generated with `openssl genrsa 512`
DKIM_PRIVATE_KEY = '''
-----BEGIN RSA PRIVATE KEY-----
MIIBOwIBAAJBALCKsjD8UUxBESo1OLN6gptp1lD0U85AgXGL571/SQ3k61KhAQ8h
hL3lnfQKn/XCl2oCXscEwgJv43IUs+VETWECAwEAAQJAQ8XK6GFEuHhWJZTu4n/K
ee0keEmDjq9WwgdKfIXLvsgaaNxCObhzv7G5rPU+U/3z1/0CtGR+DOPgoiaI/5HM
XQIhAN4h+o2WzRrz+dD/+zMGC9h1KEFvukIoP62kLOxW0eg/AiEAy3VD+UkRni4H
6UEJgCe0oZIiBCxj12/wUHFj1cfJYl8CICsndsGjFl2yIEpWMLsM5ag7uoJb7leD
8jsNthyEEWuJAiEAjeF6w26HEK286pZmD66gskN74TkrbuMqzI4mNsCZ2TUCIQCJ
HuuR7wc0HJ/cfVi8Kgm5B+sHY9/7KDWAYGGnbGgCNA==
-----END RSA PRIVATE KEY-----
'''
DKIM_PUBLIC_KEY = 'MFwwDQYJKoZIhvcNAQEBBQADSwAwSAJBALCKsjD8UUxBESo1OLN6gptp1lD0U85AgXGL571/SQ3k61KhAQ8hhL3lnfQKn/' \
                  'XCl2oCXscEwgJv43IUs+VETWECAwEAAQ=='


class SESConfigurationSetTester(object):

    def __init__(self, configuration_set):
        self.message = None
        self.dkim_domain = None
        self.dkim_key = None
        self.dkim_selector = None
        self.dkim_headers = ()
        self.configuration_set = configuration_set

    def __call__(self, message, dkim_domain=None, dkim_key=None,
                 dkim_selector=None, dkim_headers=()):
        self.message = message
        self.dkim_domain = dkim_domain
        self.dkim_key = dkim_key
        self.dkim_selector = dkim_selector
        self.dkim_headers = dkim_headers
        return self.configuration_set


class FakeSESConnection:
    """
    A fake SES connection for testing purposes.It behaves similarly
    to django's dummy backend
    (https://docs.djangoproject.com/en/dev/topics/email/#dummy-backend)

    Emails sent with send_raw_email is stored in ``outbox`` attribute
    which is a list of kwargs received by ``send_raw_email``.
    """
    outbox = []

    def __init__(self, *args, **kwargs):
        self.outbox.append(kwargs)

    def send_raw_email(self, **kwargs):
        self.outbox.append(kwargs)
        return {
            'MessageId': 'fake_message_id',
            'ResponseMetadata': {
                'RequestId': 'fake_request_id',
                },
            }

    def send_email(self, **kwargs):
        self.outbox.append(kwargs)
        return {
            'MessageId': 'fake_message_id',
            'ResponseMetadata': {
                'RequestId': 'fake_request_id',
                },
            }

    @classmethod
    def client(cls, *args, **kwargs):
        return cls(args, kwargs)


class FakeSESBackend(django_ses.SESBackend):
    """
    A fake SES backend for testing purposes. It overrides the real SESBackend's
    get_rate_limit method so we can run tests without valid AWS credentials.
    """

    def get_rate_limit(self):
        return 10

    def create_session(self) -> FakeSESConnection:
        return FakeSESConnection


class SESBackendTest(TestCase):
    def setUp(self):
        # TODO: Fix this -- this is going to cause side effects
        django_settings.EMAIL_BACKEND = "tests.test_backend.FakeSESBackend"
        self.outbox = FakeSESConnection.outbox

    def tearDown(self):
        # Empty outbox every time test finishes
        FakeSESConnection.outbox = []

    def _rfc2047_helper(self, value_to_encode):
        # references: https://docs.python.org/3/library/email.header.html, https://tools.ietf.org/html/rfc2047.html
        name, addr = email.utils.parseaddr(value_to_encode)
        name = email.header.Header(name).encode()
        return email.utils.formataddr((name, addr))

    def test_rfc2047_helper(self):
        # Ensures that the underlying email.header library code is encoding as expected, using known values
        unicode_from_addr = 'Unicode Name óóóóóó <from@example.com>'
        rfc2047_encoded_from_addr = '=?utf-8?b?VW5pY29kZSBOYW1lIMOzw7PDs8Ozw7PDsw==?= <from@example.com>'
        self.assertEqual(self._rfc2047_helper(unicode_from_addr), rfc2047_encoded_from_addr)

    def test_send_mail(self):
        settings.AWS_SES_CONFIGURATION_SET = None

        from_addr = 'Albertus Magnus <albertus.magnus@example.com>'

        send_mail('subject', 'body', from_addr, ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['RawMessage']['Data']))
        self.assertTrue('X-SES-CONFIGURATION-SET' not in mail.keys())
        self.assertEqual(mail['subject'], 'subject')
        self.assertEqual(mail['from'], self._rfc2047_helper(from_addr))
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'body')

    def test_send_mail_unicode_body(self):
        settings.AWS_SES_CONFIGURATION_SET = None

        unicode_from_addr = 'Unicode Name óóóóóó <from@example.com>'

        send_mail('Scandinavian', 'Sören & Björn', unicode_from_addr, ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['RawMessage']['Data']))
        self.assertTrue('X-SES-CONFIGURATION-SET' not in mail.keys())
        self.assertEqual(mail['subject'], 'Scandinavian')
        self.assertEqual(mail['from'], self._rfc2047_helper(unicode_from_addr))
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'Sören & Björn')

    def test_configuration_set_send_mail(self):
        settings.AWS_SES_CONFIGURATION_SET = 'test-set'
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['RawMessage']['Data']))
        self.assertEqual(mail['X-SES-CONFIGURATION-SET'], 'test-set')
        self.assertEqual(mail['subject'], 'subject')
        self.assertEqual(mail['from'], 'from@example.com')
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'body')

    def test_configuration_set_callable_send_mail(self):
        config_set_callable = SESConfigurationSetTester('my-config-set')
        settings.AWS_SES_CONFIGURATION_SET = config_set_callable
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['RawMessage']['Data']))
        # ensure we got the correct configuration message payload
        self.assertEqual(mail['X-SES-CONFIGURATION-SET'], 'my-config-set')
        self.assertEqual(mail['subject'], 'subject')
        self.assertEqual(mail['from'], 'from@example.com')
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'body')
        # ensure we passed in the proper arguments to our callable
        self.assertEqual(config_set_callable.message.subject, 'subject')
        self.assertEqual(config_set_callable.dkim_domain, None)
        self.assertEqual(config_set_callable.dkim_key, None)
        self.assertEqual(config_set_callable.dkim_selector, 'ses')
        self.assertEqual(config_set_callable.dkim_headers, ('From', 'To', 'Cc', 'Subject'))


class SESV2BackendTest(TestCase):
    def setUp(self):
        django_settings.EMAIL_BACKEND = 'tests.test_backend.FakeSESBackend'
        settings.USE_SES_V2 = True
        settings.AWS_SES_FROM_ARN = None
        settings.AWS_SES_SOURCE_ARN = None
        self.outbox = FakeSESConnection.outbox

    def tearDown(self):
        # Empty outbox every time test finishes
        settings.USE_SES_V2 = False
        FakeSESConnection.outbox = []

    def _rfc2047_helper(self, value_to_encode):
        # references: https://docs.python.org/3/library/email.header.html, https://tools.ietf.org/html/rfc2047.html
        name, addr = email.utils.parseaddr(value_to_encode)
        name = email.header.Header(name).encode()
        return email.utils.formataddr((name, addr))

    def test_rfc2047_helper(self):
        # Ensures that the underlying email.header library code is encoding as expected, using known values
        unicode_from_addr = 'Unicode Name óóóóóó <from@example.com>'
        rfc2047_encoded_from_addr = '=?utf-8?b?VW5pY29kZSBOYW1lIMOzw7PDs8Ozw7PDsw==?= <from@example.com>'
        self.assertEqual(self._rfc2047_helper(unicode_from_addr), rfc2047_encoded_from_addr)

    def test_send_mail(self):
        settings.AWS_SES_CONFIGURATION_SET = None

        unicode_from_addr = 'Unicode Name óóóóóó <from@example.com>'

        send_mail('subject', 'body', unicode_from_addr, ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['Content']['Raw']['Data']))
        self.assertTrue('X-SES-CONFIGURATION-SET' not in mail.keys())
        self.assertEqual(mail['subject'], 'subject')
        self.assertEqual(mail['from'], self._rfc2047_helper(unicode_from_addr))
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'body')

    def test_configuration_set_send_mail(self):
        settings.AWS_SES_CONFIGURATION_SET = 'test-set'
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['Content']['Raw']['Data']))
        self.assertEqual(mail['X-SES-CONFIGURATION-SET'], 'test-set')
        self.assertEqual(mail['subject'], 'subject')
        self.assertEqual(mail['from'], 'from@example.com')
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'body')

    def test_configuration_set_callable_send_mail(self):
        config_set_callable = SESConfigurationSetTester('my-config-set')
        settings.AWS_SES_CONFIGURATION_SET = config_set_callable
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        message = self.outbox.pop()
        mail = email.message_from_string(smart_str(message['Content']['Raw']['Data']))
        # ensure we got the correct configuration message payload
        self.assertEqual(mail['X-SES-CONFIGURATION-SET'], 'my-config-set')
        self.assertEqual(mail['subject'], 'subject')
        self.assertEqual(mail['from'], 'from@example.com')
        self.assertEqual(mail['to'], 'to@example.com')
        self.assertEqual(mail.get_payload(), 'body')
        # ensure we passed in the proper arguments to our callable
        self.assertEqual(config_set_callable.message.subject, 'subject')
        self.assertEqual(config_set_callable.dkim_domain, None)
        self.assertEqual(config_set_callable.dkim_key, None)
        self.assertEqual(config_set_callable.dkim_selector, 'ses')
        self.assertEqual(config_set_callable.dkim_headers, ('From', 'To', 'Cc', 'Subject'))

    def test_dkim_mail(self):
        settings.AWS_SES_CONFIGURATION_SET = None
        # DKIM verification uses DNS to retrieve the public key when checking
        # the signature, so we need to replace the standard query response with
        # one that always returns the test key.
        try:
            import dkim
            import dns
        except ImportError:
            return

        def dns_query(qname, rdtype):
            name = dns.name.from_text(qname)
            response = dns.message.from_text(
                'id 1\n;ANSWER\n%s 60 IN TXT "v=DKIM1; p=%s"' %
                (qname, DKIM_PUBLIC_KEY))
            return dns.resolver.Answer(name, rdtype, 1, response)

        dns.resolver.query = dns_query

        settings.DKIM_DOMAIN = 'example.com'
        settings.DKIM_PRIVATE_KEY = DKIM_PRIVATE_KEY
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        message = self.outbox.pop()['RawMessage']
        self.assertTrue(dkim.verify(message))
        self.assertFalse(dkim.verify(message + 'some additional text'))
        self.assertFalse(dkim.verify(
                            message.replace('from@example.com', 'from@spam.com')))

    def test_return_path(self):
        """Ensure that the 'Source' argument sent into send_raw_email uses FromEmailAddress.
        """
        settings.AWS_SES_RETURN_PATH = None
        settings.AWS_SES_FROM_EMAIL = 'from@example.com'
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        self.assertEqual(self.outbox.pop()['FromEmailAddress'], 'from@example.com')

    def test_feedback_forwarding(self):
        """
        Ensure that the notification address argument uses FeedbackForwardingEmailAddress.
        """
        settings.AWS_SES_RETURN_PATH = 'reply@example.com'
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        self.assertEqual(self.outbox.pop()['FeedbackForwardingEmailAddress'], 'reply@example.com')

    def test_source_arn_is_not_set(self):
        """
        Ensure that the helpers for Identity Owner for SES Sending Authorization are not present, if nothing has been
        configured.
        """
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        mail = self.outbox.pop()
        self.assertNotIn('FromEmailAddressIdentityArn', mail)

    def test_source_arn_is_set(self):
        """
        Ensure that the helpers for Identity Owner for SES Sending Authorization are set correctly.
        """
        settings.AWS_SES_SOURCE_ARN = 'arn:aws:ses:eu-central-1:111111111111:identity/example.com'
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        mail = self.outbox.pop()
        self.assertEqual(mail['FromEmailAddressIdentityArn'],
                         'arn:aws:ses:eu-central-1:111111111111:identity/example.com')

    def test_from_arn_takes_precedence_when_source_arn_is_set(self):
        """
        Ensure that the helpers for Identity Owner for SES Sending Authorization are set correctly.
        """
        settings.AWS_SES_SOURCE_ARN = 'arn:aws:ses:eu-central-1:111111111111:identity/example.com'
        settings.AWS_SES_FROM_ARN = 'arn:aws:ses:eu-central-1:222222222222:identity/example.com'
        settings.AWS_SES_RETURN_PATH_ARN = 'arn:aws:ses:eu-central-1:333333333333:identity/example.com'
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        mail = self.outbox.pop()
        self.assertEqual(mail['FromEmailAddressIdentityArn'],
                         'arn:aws:ses:eu-central-1:222222222222:identity/example.com')


class SESBackendTestInitialize(TestCase):
    def test_auto_throttle(self):
        """
        Ensure that SESBackend handles aws_auto_throttle correctly
        """
        for throttle_param, throttle_setting, expected_throttle_val in (
            # If provided, we should cast parameter to float
            (1.0, 2.0, 1.0),
            ("1.0", 2.0, 1.0),

            # On 0 or None, we should fall back to the value in Django settings,
            # casting that value to a float
            (0, 2.0, 2.0),
            (None, 2.0, 2.0),
            (None, "1.0", 1.0),
            (None, None, None),
        ):
            settings.AWS_SES_AUTO_THROTTLE = throttle_setting
            backend = django_ses.SESBackend(aws_auto_throttle=throttle_param)
            self.assertEqual(backend._throttle, expected_throttle_val)


class SESBackendTestReturn(TestCase):
    def setUp(self):
        # TODO: Fix this -- this is going to cause side effects
        django_settings.EMAIL_BACKEND = 'tests.test_backend.FakeSESBackend'
        django_ses.SESConnection = FakeSESConnection
        self.outbox = FakeSESConnection.outbox

    def tearDown(self):
        # Empty outbox everytime test finishes
        FakeSESConnection.outbox = []

    def test_from_email(self):
        settings.AWS_SES_FROM_EMAIL = "my_default_from@example.com"
        send_mail('subject', 'body', 'ignored_from@example.com', ['to@example.com'])
        self.assertEqual(self.outbox.pop()['Source'], 'my_default_from@example.com')

    def test_return_path(self):
        settings.USE_SES_V2 = True
        settings.AWS_SES_RETURN_PATH = "return@example.com"
        send_mail('subject', 'body', 'from@example.com', ['to@example.com'])
        message = self.outbox.pop()

        self.assertEqual(message['FromEmailAddress'], 'my_default_from@example.com')
        self.assertEqual(message['FeedbackForwardingEmailAddress'], 'return@example.com')
